#ifndef __mm1EXT_H__
#define __mm1EXT_H__

#include "Cpp/BaseClasses/Modest/bool_fun.h"
#include "Cpp/BaseClasses/Modest/ModestGuard.h"
#include "Cpp/BaseClasses/Modest/ModestConstruct.h"
#include "Cpp/BaseClasses/Modest/ModestAction.h"
#include "Cpp/BaseClasses/Modest/ModestSyncAction.h"
#include "Cpp/BaseClasses/Modest/ModestVariable.h"
#include "Cpp/BaseClasses/Modest/ModestStructVariable.h"
#include "Cpp/BaseClasses/Modest/ModestArrayVariable.h"
#include "Cpp/BaseClasses/Modest/ModestModel.h"
#include "Cpp/BaseClasses/Modest/StateBit.h"
#include "Cpp/BaseClasses/Modest/TTDReader.h"

// Type-definitions for user defined types


// Model Parameters

class mm1EXT : public ModestModel {
public:
	typedef ModestModel::bitvector_type bitvector_type;

	typedef const ModestModel::bitvector_type mask_type[16][1];

	static mask_type *bitmasks;

	virtual const bitvector_type* getBitMask( size_t idx ) const {
		return &((*bitmasks)[idx][0]);
	}

	// Structure used for state save/restore
	typedef struct {
		bitvector_type enabling_bits[1];
		int	n_queue1;
		int	ql_queue1;
		double	wait_consumer1;
		double	wait_producer1;
		double	y_producer1;
		double	z_consumer1;
	} StateStruct;

	virtual int StateSize() {
		return sizeof(StateStruct);
	}

	virtual void CurrentState( void* memory );
	virtual void SetState( void* memory );
	virtual void printState( void ) { printState(&std::cout); } ;
	virtual void printState( std::ostream *strm );

	virtual bool checkEnablingBits( unsigned int offset, bitvector_type mask ) const
	{
		return ((enabling_bits[offset] & mask) != 0);
	}

	virtual bitvector_type* getEnablingBits()
	{
		return enabling_bits;
	}

	ModestConstruct* getTermTree( void ) const {
		return term_tree;
	}


	// Constructor for the model
	mm1EXT();

	// Constructor for the model
	virtual ~mm1EXT();
public:
	ModestConstruct** con;		// array for construct tree
	ModestConstruct* term_tree;		// root of the construct tree


	// Attributes for variables
	ModestVariable<int>* n_queue1;
	ModestVariable<int>* ql_queue1;
	ModestVariable<double>* wait_consumer1;
	ModestVariable<double>* wait_producer1;
	ModestVariable<double>* y_producer1;
	ModestVariable<double>* z_consumer1;

	// Attributes for the actions in the model
	ModestSyncAction* getSync0;
	ModestAction* get_16;
	ModestAction* get_26;
	ModestSyncAction* putSync1;
	ModestAction* put_9;
	ModestAction* put_19;
	ModestSyncAction* putSync2;
	ModestAction* put_22;
	ModestAction* tau_27;
	ModestAction* tau_31;
	ModestAction* tau_12;
	ModestAction* tau_5;

	// StateVariables for the static state bits
	ModestStateBit bit0;
	ModestStateBit bit1;
	ModestStateBit bit2;
	ModestStateBit bit3;
	ModestStateBit bit4;
	ModestStateBit bit5;
	ModestStateBit bit6;
	ModestStateBit bit7;
	ModestStateBit bit8;

	// Attributes for guards
	ModestGuard* guard7;
	ModestGuard* guard8;
	ModestGuard* guard15;
	ModestGuard* guard18;
	ModestGuard* guard21;
	ModestGuard* guard29;
	ModestGuard* guard30;

	bitvector_type enabling_bits[1];
}; // end ModestModel mm1
#endif
