#ifndef __rdt2_1EXT_H__
#define __rdt2_1EXT_H__

#include "Cpp/BaseClasses/Modest/bool_fun.h"
#include "Cpp/BaseClasses/Modest/ModestGuard.h"
#include "Cpp/BaseClasses/Modest/ModestConstruct.h"
#include "Cpp/BaseClasses/Modest/ModestAction.h"
#include "Cpp/BaseClasses/Modest/ModestSyncAction.h"
#include "Cpp/BaseClasses/Modest/ModestVariable.h"
#include "Cpp/BaseClasses/Modest/ModestStructVariable.h"
#include "Cpp/BaseClasses/Modest/ModestArrayVariable.h"
#include "Cpp/BaseClasses/Modest/ModestModel.h"
#include "Cpp/BaseClasses/Modest/StateBit.h"
#include "Cpp/BaseClasses/Modest/TTDReader.h"

// Type-definitions for user defined types

struct state_PACKET
{
	int seq_num;
	int type;
	int status;
};

template <typename CT, typename T>
std::basic_ostream<CT,T>& operator<<( std::basic_ostream<CT,T>& s, const state_PACKET& a )
{
	s	<< ".seq_num = " << a.seq_num << std::endl
		<< ".type = " << a.type << std::endl
		<< ".status = " << a.status << std::endl
		;
	return s;
}

class PACKET : public ModestStructVariable<state_PACKET>
{
public:
	ModestVariable<int>* seq_num;
	ModestVariable<int>* type;
	ModestVariable<int>* status;
	PACKET( char* name );
};


// Model Parameters

class rdt2_1EXT : public ModestModel {
public:
	typedef ModestModel::bitvector_type bitvector_type;

	typedef const ModestModel::bitvector_type mask_type[77][2];

	static mask_type *bitmasks;

	virtual const bitvector_type* getBitMask( size_t idx ) const {
		return &((*bitmasks)[idx][0]);
	}

	// Structure used for state save/restore
	typedef struct {
		bitvector_type enabling_bits[2];
		state_PACKET	p_receiver1;
		state_PACKET	p_sender1;
		state_PACKET	packet;
		int	resend_0;
		int	resend_1;
	} StateStruct;

	virtual int StateSize() {
		return sizeof(StateStruct);
	}

	virtual void CurrentState( void* memory );
	virtual void SetState( void* memory );
	virtual void printState( void ) { printState(&std::cout); } ;
	virtual void printState( std::ostream *strm );

	virtual bool checkEnablingBits( unsigned int offset, bitvector_type mask ) const
	{
		return ((enabling_bits[offset] & mask) != 0);
	}

	virtual bitvector_type* getEnablingBits()
	{
		return enabling_bits;
	}

	ModestConstruct* getTermTree( void ) const {
		return term_tree;
	}


	// Constructor for the model
	rdt2_1EXT();

	// Constructor for the model
	virtual ~rdt2_1EXT();
public:
	ModestConstruct** con;		// array for construct tree
	ModestConstruct* term_tree;		// root of the construct tree


	// Attributes for variables
	PACKET* p_receiver1;
	PACKET* p_sender1;
	PACKET* packet;
	ModestVariable<int>* resend_0;
	ModestVariable<int>* resend_1;

	// Attributes for the actions in the model
	ModestAction* break_25;
	ModestAction* break_41;
	ModestAction* break_67;
	ModestAction* break_85;
	ModestAction* deliver_data_62;
	ModestAction* deliver_data_80;
	ModestAction* extract_61;
	ModestAction* extract_79;
	ModestSyncAction* rdt_rcv_rSync0;
	ModestAction* rdt_rcv_52;
	ModestAction* rdt_rcv_r_91;
	ModestSyncAction* rdt_rcv_rSync1;
	ModestAction* rdt_rcv_r_92;
	ModestSyncAction* rdt_rcv_rSync2;
	ModestAction* rdt_rcv_r_93;
	ModestSyncAction* rdt_rcv_rSync3;
	ModestAction* rdt_rcv_70;
	ModestSyncAction* rdt_rcv_rSync4;
	ModestSyncAction* rdt_rcv_rSync5;
	ModestSyncAction* rdt_rcv_sSync6;
	ModestAction* rdt_rcv_17;
	ModestAction* rdt_rcv_s_97;
	ModestSyncAction* rdt_rcv_sSync7;
	ModestAction* rdt_rcv_s_98;
	ModestSyncAction* rdt_rcv_sSync8;
	ModestAction* rdt_rcv_s_99;
	ModestSyncAction* rdt_rcv_sSync9;
	ModestAction* rdt_rcv_33;
	ModestSyncAction* rdt_rcv_sSync10;
	ModestSyncAction* rdt_rcv_sSync11;
	ModestAction* rdt_send_10;
	ModestAction* rdt_send_26;
	ModestAction* tau_90;
	ModestAction* tau_96;
	ModestAction* compute_checksum_11;
	ModestAction* compute_checksum_27;
	ModestAction* make_pkt_12;
	ModestAction* make_pkt_28;
	ModestAction* tau_6;
	ModestAction* tau_8;
	ModestAction* tau_21;
	ModestAction* tau_37;
	ModestAction* compute_checksum_63;
	ModestAction* compute_checksum_81;
	ModestAction* make_pkt_64;
	ModestAction* make_pkt_82;
	ModestAction* tau_46;
	ModestAction* tau_48;
	ModestAction* tau_56;
	ModestAction* tau_74;
	ModestSyncAction* udt_send_rSync12;
	ModestAction* udt_send_58;
	ModestAction* udt_send_r_95;
	ModestSyncAction* udt_send_rSync13;
	ModestAction* udt_send_66;
	ModestSyncAction* udt_send_rSync14;
	ModestAction* udt_send_76;
	ModestSyncAction* udt_send_rSync15;
	ModestAction* udt_send_84;
	ModestSyncAction* udt_send_sSync16;
	ModestAction* udt_send_14;
	ModestAction* udt_send_s_89;
	ModestSyncAction* udt_send_sSync17;
	ModestAction* udt_send_23;
	ModestSyncAction* udt_send_sSync18;
	ModestAction* udt_send_30;
	ModestSyncAction* udt_send_sSync19;
	ModestAction* udt_send_39;

	// StateVariables for the static state bits
	ModestStateBit bit0;
	ModestStateBit bit1;
	ModestStateBit bit2;
	ModestStateBit bit3;
	ModestStateBit bit4;
	ModestStateBit bit5;
	ModestStateBit bit6;
	ModestStateBit bit7;
	ModestStateBit bit8;
	ModestStateBit bit9;
	ModestStateBit bit10;
	ModestStateBit bit11;
	ModestStateBit bit12;
	ModestStateBit bit13;
	ModestStateBit bit14;
	ModestStateBit bit15;
	ModestStateBit bit16;
	ModestStateBit bit17;
	ModestStateBit bit18;
	ModestStateBit bit19;
	ModestStateBit bit20;
	ModestStateBit bit21;
	ModestStateBit bit22;
	ModestStateBit bit23;
	ModestStateBit bit24;
	ModestStateBit bit25;
	ModestStateBit bit26;
	ModestStateBit bit27;
	ModestStateBit bit28;
	ModestStateBit bit29;
	ModestStateBit bit30;
	ModestStateBit bit31;
	ModestStateBit bit32;
	ModestStateBit bit33;
	ModestStateBit bit34;
	ModestStateBit bit35;
	ModestStateBit bit36;
	ModestStateBit bit37;
	ModestStateBit bit38;
	ModestStateBit bit39;
	ModestStateBit bit40;
	ModestStateBit bit41;
	ModestStateBit bit42;
	ModestStateBit bit43;
	ModestStateBit bit44;
	ModestStateBit bit45;
	ModestStateBit bit46;
	ModestStateBit bit47;

	// Attributes for guards
	ModestGuard* guard20;
	ModestGuard* guard24;
	ModestGuard* guard36;
	ModestGuard* guard40;
	ModestGuard* guard55;
	ModestGuard* guard60;
	ModestGuard* guard73;
	ModestGuard* guard78;

	bitvector_type enabling_bits[2];
}; // end ModestModel rdt2_1
#endif
