#ifndef __rdt3_1EXT_H__
#define __rdt3_1EXT_H__

#include "Cpp/BaseClasses/Modest/bool_fun.h"
#include "Cpp/BaseClasses/Modest/ModestGuard.h"
#include "Cpp/BaseClasses/Modest/ModestConstruct.h"
#include "Cpp/BaseClasses/Modest/ModestAction.h"
#include "Cpp/BaseClasses/Modest/ModestSyncAction.h"
#include "Cpp/BaseClasses/Modest/ModestVariable.h"
#include "Cpp/BaseClasses/Modest/ModestStructVariable.h"
#include "Cpp/BaseClasses/Modest/ModestArrayVariable.h"
#include "Cpp/BaseClasses/Modest/ModestModel.h"
#include "Cpp/BaseClasses/Modest/StateBit.h"
#include "Cpp/BaseClasses/Modest/TTDReader.h"

// Type-definitions for user defined types

struct state_PACKET
{
	int seq_num;
	int type;
	int status;
};

template <typename CT, typename T>
std::basic_ostream<CT,T>& operator<<( std::basic_ostream<CT,T>& s, const state_PACKET& a )
{
	s	<< ".seq_num = " << a.seq_num << std::endl
		<< ".type = " << a.type << std::endl
		<< ".status = " << a.status << std::endl
		;
	return s;
}

class PACKET : public ModestStructVariable<state_PACKET>
{
public:
	ModestVariable<int>* seq_num;
	ModestVariable<int>* type;
	ModestVariable<int>* status;
	PACKET( char* name );
};


// Model Parameters
extern double lprop;
extern double timeout;
extern double uprop;

class rdt3_1EXT : public ModestModel {
public:
	typedef ModestModel::bitvector_type bitvector_type;

	typedef const ModestModel::bitvector_type mask_type[110][3];

	static mask_type *bitmasks;

	virtual const bitvector_type* getBitMask( size_t idx ) const {
		return &((*bitmasks)[idx][0]);
	}

	// Structure used for state save/restore
	typedef struct {
		bitvector_type enabling_bits[3];
		state_PACKET	p_receiver1;
		state_PACKET	p_sender1;
		state_PACKET	packet;
		double	prop_channel1;
		double	sendtime0;
		double	sendtime1;
		double	x_channel1;
		double	x_sender1;
	} StateStruct;

	virtual int StateSize() {
		return sizeof(StateStruct);
	}

	virtual void CurrentState( void* memory );
	virtual void SetState( void* memory );
	virtual void printState( void ) { printState(&std::cout); } ;
	virtual void printState( std::ostream *strm );

	virtual bool checkEnablingBits( unsigned int offset, bitvector_type mask ) const
	{
		return ((enabling_bits[offset] & mask) != 0);
	}

	virtual bitvector_type* getEnablingBits()
	{
		return enabling_bits;
	}

	ModestConstruct* getTermTree( void ) const {
		return term_tree;
	}


	// Constructor for the model
	rdt3_1EXT();

	// Constructor for the model
	virtual ~rdt3_1EXT();
public:
	ModestConstruct** con;		// array for construct tree
	ModestConstruct* term_tree;		// root of the construct tree


	// Attributes for variables
	PACKET* p_receiver1;
	PACKET* p_sender1;
	PACKET* packet;
	ModestVariable<double>* prop_channel1;
	ModestVariable<double>* sendtime0;
	ModestVariable<double>* sendtime1;
	ModestVariable<double>* x_channel1;
	ModestVariable<double>* x_sender1;

	// Attributes for the actions in the model
	ModestAction* break_14;
	ModestAction* break_37;
	ModestAction* break_50;
	ModestAction* break_73;
	ModestAction* break_109;
	ModestAction* break_127;
	ModestAction* deliver_data_104;
	ModestAction* deliver_data_122;
	ModestAction* extract_103;
	ModestAction* extract_121;
	ModestSyncAction* rdt_rcv_rSync0;
	ModestAction* rdt_rcv_94;
	ModestAction* rdt_rcv_r_136;
	ModestSyncAction* rdt_rcv_rSync1;
	ModestAction* rdt_rcv_r_137;
	ModestSyncAction* rdt_rcv_rSync2;
	ModestAction* rdt_rcv_r_138;
	ModestSyncAction* rdt_rcv_rSync3;
	ModestAction* rdt_rcv_112;
	ModestSyncAction* rdt_rcv_rSync4;
	ModestSyncAction* rdt_rcv_rSync5;
	ModestSyncAction* rdt_rcv_sSync6;
	ModestAction* rdt_rcv_11;
	ModestAction* rdt_rcv_s_146;
	ModestSyncAction* rdt_rcv_sSync7;
	ModestAction* rdt_rcv_s_147;
	ModestSyncAction* rdt_rcv_sSync8;
	ModestAction* rdt_rcv_s_148;
	ModestSyncAction* rdt_rcv_sSync9;
	ModestAction* rdt_rcv_24;
	ModestSyncAction* rdt_rcv_sSync10;
	ModestSyncAction* rdt_rcv_sSync11;
	ModestSyncAction* rdt_rcv_sSync12;
	ModestAction* rdt_rcv_47;
	ModestSyncAction* rdt_rcv_sSync13;
	ModestSyncAction* rdt_rcv_sSync14;
	ModestSyncAction* rdt_rcv_sSync15;
	ModestAction* rdt_rcv_60;
	ModestSyncAction* rdt_rcv_sSync16;
	ModestSyncAction* rdt_rcv_sSync17;
	ModestAction* rdt_send_13;
	ModestAction* rdt_send_49;
	ModestAction* start_timer_19;
	ModestAction* start_timer_31;
	ModestAction* start_timer_44;
	ModestAction* start_timer_55;
	ModestAction* start_timer_67;
	ModestAction* start_timer_80;
	ModestAction* tau_135;
	ModestAction* tau_139;
	ModestAction* tau_145;
	ModestAction* tau_149;
	ModestAction* compute_checksum_15;
	ModestAction* compute_checksum_51;
	ModestAction* make_pkt_16;
	ModestAction* make_pkt_52;
	ModestAction* tau_6;
	ModestAction* tau_8;
	ModestAction* tau_28;
	ModestAction* tau_35;
	ModestAction* tau_41;
	ModestAction* tau_64;
	ModestAction* tau_71;
	ModestAction* tau_77;
	ModestAction* compute_checksum_105;
	ModestAction* compute_checksum_123;
	ModestAction* make_pkt_106;
	ModestAction* make_pkt_124;
	ModestAction* tau_86;
	ModestAction* tau_88;
	ModestAction* tau_98;
	ModestAction* tau_116;
	ModestSyncAction* udt_send_rSync18;
	ModestAction* udt_send_100;
	ModestAction* udt_send_r_141;
	ModestSyncAction* udt_send_rSync19;
	ModestAction* udt_send_108;
	ModestSyncAction* udt_send_rSync20;
	ModestAction* udt_send_118;
	ModestSyncAction* udt_send_rSync21;
	ModestAction* udt_send_126;
	ModestSyncAction* udt_send_sSync22;
	ModestAction* udt_send_18;
	ModestAction* udt_send_s_131;
	ModestSyncAction* udt_send_sSync23;
	ModestAction* udt_send_30;
	ModestSyncAction* udt_send_sSync24;
	ModestAction* udt_send_43;
	ModestSyncAction* udt_send_sSync25;
	ModestAction* udt_send_54;
	ModestSyncAction* udt_send_sSync26;
	ModestAction* udt_send_66;
	ModestSyncAction* udt_send_sSync27;
	ModestAction* udt_send_79;

	// StateVariables for the static state bits
	ModestStateBit bit0;
	ModestStateBit bit1;
	ModestStateBit bit2;
	ModestStateBit bit3;
	ModestStateBit bit4;
	ModestStateBit bit5;
	ModestStateBit bit6;
	ModestStateBit bit7;
	ModestStateBit bit8;
	ModestStateBit bit9;
	ModestStateBit bit10;
	ModestStateBit bit11;
	ModestStateBit bit12;
	ModestStateBit bit13;
	ModestStateBit bit14;
	ModestStateBit bit15;
	ModestStateBit bit16;
	ModestStateBit bit17;
	ModestStateBit bit18;
	ModestStateBit bit19;
	ModestStateBit bit20;
	ModestStateBit bit21;
	ModestStateBit bit22;
	ModestStateBit bit23;
	ModestStateBit bit24;
	ModestStateBit bit25;
	ModestStateBit bit26;
	ModestStateBit bit27;
	ModestStateBit bit28;
	ModestStateBit bit29;
	ModestStateBit bit30;
	ModestStateBit bit31;
	ModestStateBit bit32;
	ModestStateBit bit33;
	ModestStateBit bit34;
	ModestStateBit bit35;
	ModestStateBit bit36;
	ModestStateBit bit37;
	ModestStateBit bit38;
	ModestStateBit bit39;
	ModestStateBit bit40;
	ModestStateBit bit41;
	ModestStateBit bit42;
	ModestStateBit bit43;
	ModestStateBit bit44;
	ModestStateBit bit45;
	ModestStateBit bit46;
	ModestStateBit bit47;
	ModestStateBit bit48;
	ModestStateBit bit49;
	ModestStateBit bit50;
	ModestStateBit bit51;
	ModestStateBit bit52;
	ModestStateBit bit53;
	ModestStateBit bit54;
	ModestStateBit bit55;
	ModestStateBit bit56;
	ModestStateBit bit57;
	ModestStateBit bit58;
	ModestStateBit bit59;
	ModestStateBit bit60;
	ModestStateBit bit61;
	ModestStateBit bit62;
	ModestStateBit bit63;
	ModestStateBit bit64;
	ModestStateBit bit65;

	// Attributes for guards
	ModestGuard* guard23;
	ModestGuard* guard27;
	ModestGuard* guard34;
	ModestGuard* guard39;
	ModestGuard* guard40;
	ModestGuard* guard59;
	ModestGuard* guard63;
	ModestGuard* guard70;
	ModestGuard* guard75;
	ModestGuard* guard76;
	ModestGuard* guard97;
	ModestGuard* guard102;
	ModestGuard* guard115;
	ModestGuard* guard120;
	ModestGuard* guard133;
	ModestGuard* guard134;
	ModestGuard* guard143;
	ModestGuard* guard144;

	bitvector_type enabling_bits[3];
}; // end ModestModel rdt3_1
#endif
