#ifndef __rdt4_0EXT_H__
#define __rdt4_0EXT_H__

#include "Cpp/BaseClasses/Modest/bool_fun.h"
#include "Cpp/BaseClasses/Modest/ModestGuard.h"
#include "Cpp/BaseClasses/Modest/ModestConstruct.h"
#include "Cpp/BaseClasses/Modest/ModestAction.h"
#include "Cpp/BaseClasses/Modest/ModestSyncAction.h"
#include "Cpp/BaseClasses/Modest/ModestVariable.h"
#include "Cpp/BaseClasses/Modest/ModestStructVariable.h"
#include "Cpp/BaseClasses/Modest/ModestArrayVariable.h"
#include "Cpp/BaseClasses/Modest/ModestModel.h"
#include "Cpp/BaseClasses/Modest/StateBit.h"
#include "Cpp/BaseClasses/Modest/TTDReader.h"

// Type-definitions for user defined types

struct state_PACKET
{
	int seq_num;
	int type;
	int status;
};

template <typename CT, typename T>
std::basic_ostream<CT,T>& operator<<( std::basic_ostream<CT,T>& s, const state_PACKET& a )
{
	s	<< ".seq_num = " << a.seq_num << std::endl
		<< ".type = " << a.type << std::endl
		<< ".status = " << a.status << std::endl
		;
	return s;
}

class PACKET : public ModestStructVariable<state_PACKET>
{
public:
	ModestVariable<int>* seq_num;
	ModestVariable<int>* type;
	ModestVariable<int>* status;
	PACKET( char* name );
};


// Model Parameters
extern double lprop;
extern double timeout;
extern double uprop;

class rdt4_0EXT : public ModestModel {
public:
	typedef ModestModel::bitvector_type bitvector_type;

	typedef const ModestModel::bitvector_type mask_type[127][3];

	static mask_type *bitmasks;

	virtual const bitvector_type* getBitMask( size_t idx ) const {
		return &((*bitmasks)[idx][0]);
	}

	// Structure used for state save/restore
	typedef struct {
		bitvector_type enabling_bits[3];
		state_PACKET	pa_sender1;
		state_PACKET	pa_sender2;
		state_PACKET	packet;
		int	pctc1;
		int	pctl1;
		int	pcto1;
		int	pcts1;
		state_PACKET	pd_sender1;
		state_PACKET	pd_sender2;
		double	prop_channel1;
		int	snd_i_sender1;
		int	snd_i_sender2;
		int	snd_o_sender1;
		int	snd_o_sender2;
		double	x_channel1;
		double	x_sender1;
		double	x_sender2;
	} StateStruct;

	virtual int StateSize() {
		return sizeof(StateStruct);
	}

	virtual void CurrentState( void* memory );
	virtual void SetState( void* memory );
	virtual void printState( void ) { printState(&std::cout); } ;
	virtual void printState( std::ostream *strm );

	virtual bool checkEnablingBits( unsigned int offset, bitvector_type mask ) const
	{
		return ((enabling_bits[offset] & mask) != 0);
	}

	virtual bitvector_type* getEnablingBits()
	{
		return enabling_bits;
	}

	ModestConstruct* getTermTree( void ) const {
		return term_tree;
	}


	// Constructor for the model
	rdt4_0EXT();

	// Constructor for the model
	virtual ~rdt4_0EXT();
public:
	ModestConstruct** con;		// array for construct tree
	ModestConstruct* term_tree;		// root of the construct tree


	// Attributes for variables
	PACKET* pa_sender1;
	PACKET* pa_sender2;
	PACKET* packet;
	ModestVariable<int>* pctc1;
	ModestVariable<int>* pctl1;
	ModestVariable<int>* pcto1;
	ModestVariable<int>* pcts1;
	PACKET* pd_sender1;
	PACKET* pd_sender2;
	ModestVariable<double>* prop_channel1;
	ModestVariable<int>* snd_i_sender1;
	ModestVariable<int>* snd_i_sender2;
	ModestVariable<int>* snd_o_sender1;
	ModestVariable<int>* snd_o_sender2;
	ModestVariable<double>* x_channel1;
	ModestVariable<double>* x_sender1;
	ModestVariable<double>* x_sender2;

	// Attributes for the actions in the model
	ModestAction* break_34;
	ModestAction* break_57;
	ModestAction* break_108;
	ModestAction* break_131;
	ModestSyncAction* rdt_rcv_1Sync0;
	ModestAction* rdt_rcv_15;
	ModestAction* rdt_rcv_1_171;
	ModestSyncAction* rdt_rcv_1Sync1;
	ModestAction* rdt_rcv_1_172;
	ModestSyncAction* rdt_rcv_1Sync2;
	ModestAction* rdt_rcv_1_173;
	ModestSyncAction* rdt_rcv_1Sync3;
	ModestAction* rdt_rcv_44;
	ModestSyncAction* rdt_rcv_1Sync4;
	ModestSyncAction* rdt_rcv_1Sync5;
	ModestSyncAction* rdt_rcv_2Sync6;
	ModestAction* rdt_rcv_89;
	ModestAction* rdt_rcv_2_161;
	ModestSyncAction* rdt_rcv_2Sync7;
	ModestAction* rdt_rcv_2_162;
	ModestSyncAction* rdt_rcv_2Sync8;
	ModestAction* rdt_rcv_2_163;
	ModestSyncAction* rdt_rcv_2Sync9;
	ModestAction* rdt_rcv_118;
	ModestSyncAction* rdt_rcv_2Sync10;
	ModestSyncAction* rdt_rcv_2Sync11;
	ModestAction* tau_2;
	ModestAction* tau_160;
	ModestAction* tau_164;
	ModestAction* tau_170;
	ModestAction* tau_174;
	ModestAction* compute_checksum_26;
	ModestAction* compute_checksum_35;
	ModestAction* compute_checksum_67;
	ModestAction* deliver_data_25;
	ModestAction* deliver_data_66;
	ModestAction* extract_24;
	ModestAction* extract_65;
	ModestAction* make_pkt_27;
	ModestAction* make_pkt_36;
	ModestAction* make_pkt_68;
	ModestAction* rdt_send_33;
	ModestAction* start_timer_39;
	ModestAction* start_timer_51;
	ModestAction* start_timer_77;
	ModestAction* tau_9;
	ModestAction* tau_19;
	ModestAction* tau_31;
	ModestAction* tau_48;
	ModestAction* tau_55;
	ModestAction* tau_60;
	ModestAction* tau_74;
	ModestAction* compute_checksum_100;
	ModestAction* compute_checksum_109;
	ModestAction* compute_checksum_141;
	ModestAction* deliver_data_99;
	ModestAction* deliver_data_140;
	ModestAction* extract_98;
	ModestAction* extract_139;
	ModestAction* make_pkt_101;
	ModestAction* make_pkt_110;
	ModestAction* make_pkt_142;
	ModestAction* rdt_send_107;
	ModestAction* start_timer_113;
	ModestAction* start_timer_125;
	ModestAction* start_timer_151;
	ModestAction* tau_83;
	ModestAction* tau_93;
	ModestAction* tau_105;
	ModestAction* tau_122;
	ModestAction* tau_129;
	ModestAction* tau_134;
	ModestAction* tau_148;
	ModestSyncAction* udt_send_1Sync12;
	ModestAction* udt_send_21;
	ModestAction* udt_send_1_156;
	ModestSyncAction* udt_send_1Sync13;
	ModestAction* udt_send_29;
	ModestSyncAction* udt_send_1Sync14;
	ModestAction* udt_send_38;
	ModestSyncAction* udt_send_1Sync15;
	ModestAction* udt_send_50;
	ModestSyncAction* udt_send_1Sync16;
	ModestAction* udt_send_62;
	ModestSyncAction* udt_send_1Sync17;
	ModestAction* udt_send_70;
	ModestSyncAction* udt_send_1Sync18;
	ModestAction* udt_send_76;
	ModestSyncAction* udt_send_2Sync19;
	ModestAction* udt_send_95;
	ModestAction* udt_send_2_166;
	ModestSyncAction* udt_send_2Sync20;
	ModestAction* udt_send_103;
	ModestSyncAction* udt_send_2Sync21;
	ModestAction* udt_send_112;
	ModestSyncAction* udt_send_2Sync22;
	ModestAction* udt_send_124;
	ModestSyncAction* udt_send_2Sync23;
	ModestAction* udt_send_136;
	ModestSyncAction* udt_send_2Sync24;
	ModestAction* udt_send_144;
	ModestSyncAction* udt_send_2Sync25;
	ModestAction* udt_send_150;

	// StateVariables for the static state bits
	ModestStateBit bit0;
	ModestStateBit bit1;
	ModestStateBit bit2;
	ModestStateBit bit3;
	ModestStateBit bit4;
	ModestStateBit bit5;
	ModestStateBit bit6;
	ModestStateBit bit7;
	ModestStateBit bit8;
	ModestStateBit bit9;
	ModestStateBit bit10;
	ModestStateBit bit11;
	ModestStateBit bit12;
	ModestStateBit bit13;
	ModestStateBit bit14;
	ModestStateBit bit15;
	ModestStateBit bit16;
	ModestStateBit bit17;
	ModestStateBit bit18;
	ModestStateBit bit19;
	ModestStateBit bit20;
	ModestStateBit bit21;
	ModestStateBit bit22;
	ModestStateBit bit23;
	ModestStateBit bit24;
	ModestStateBit bit25;
	ModestStateBit bit26;
	ModestStateBit bit27;
	ModestStateBit bit28;
	ModestStateBit bit29;
	ModestStateBit bit30;
	ModestStateBit bit31;
	ModestStateBit bit32;
	ModestStateBit bit33;
	ModestStateBit bit34;
	ModestStateBit bit35;
	ModestStateBit bit36;
	ModestStateBit bit37;
	ModestStateBit bit38;
	ModestStateBit bit39;
	ModestStateBit bit40;
	ModestStateBit bit41;
	ModestStateBit bit42;
	ModestStateBit bit43;
	ModestStateBit bit44;
	ModestStateBit bit45;
	ModestStateBit bit46;
	ModestStateBit bit47;
	ModestStateBit bit48;
	ModestStateBit bit49;
	ModestStateBit bit50;
	ModestStateBit bit51;
	ModestStateBit bit52;
	ModestStateBit bit53;
	ModestStateBit bit54;
	ModestStateBit bit55;
	ModestStateBit bit56;
	ModestStateBit bit57;
	ModestStateBit bit58;
	ModestStateBit bit59;
	ModestStateBit bit60;
	ModestStateBit bit61;
	ModestStateBit bit62;
	ModestStateBit bit63;
	ModestStateBit bit64;
	ModestStateBit bit65;
	ModestStateBit bit66;
	ModestStateBit bit67;
	ModestStateBit bit68;
	ModestStateBit bit69;
	ModestStateBit bit70;
	ModestStateBit bit71;
	ModestStateBit bit72;
	ModestStateBit bit73;
	ModestStateBit bit74;
	ModestStateBit bit75;
	ModestStateBit bit76;

	// Attributes for guards
	ModestGuard* guard18;
	ModestGuard* guard23;
	ModestGuard* guard30;
	ModestGuard* guard43;
	ModestGuard* guard47;
	ModestGuard* guard54;
	ModestGuard* guard59;
	ModestGuard* guard64;
	ModestGuard* guard72;
	ModestGuard* guard73;
	ModestGuard* guard92;
	ModestGuard* guard97;
	ModestGuard* guard104;
	ModestGuard* guard117;
	ModestGuard* guard121;
	ModestGuard* guard128;
	ModestGuard* guard133;
	ModestGuard* guard138;
	ModestGuard* guard146;
	ModestGuard* guard147;
	ModestGuard* guard158;
	ModestGuard* guard159;
	ModestGuard* guard168;
	ModestGuard* guard169;

	bitvector_type enabling_bits[3];
}; // end ModestModel rdt4_0
#endif
