
#include "Study/study_rdt3_0/study_rdt3_0SetStudy.h"

//******************************************************
//Global Variables
//******************************************************
double lprop;
double timeout;
double uprop;

//********************************************************
//study_rdt3_0SetStudy Constructor
//********************************************************
study_rdt3_0SetStudy::study_rdt3_0SetStudy() {

  // define arrays of global variable names and types
  NumGVs = 3;
  NumExps = 2;

  GVNames = new char*[NumGVs];
  GVTypes = new char*[NumGVs];
  GVNames[0]=strdup("lprop");
  GVTypes[0]=strdup("double");
  GVNames[1]=strdup("timeout");
  GVTypes[1]=strdup("double");
  GVNames[2]=strdup("uprop");
  GVTypes[2]=strdup("double");

  // create the arrays to store the values of each gv
  lpropValues = new double[NumExps];
  timeoutValues = new double[NumExps];
  upropValues = new double[NumExps];

  // call methods to assign values to each gv
  SetValues_lprop();
  SetValues_timeout();
  SetValues_uprop();
  SetDefaultMobiusRoot(MOBIUSROOT);
}


//******************************************************
//study_rdt3_0SetStudy Destructor
//******************************************************
study_rdt3_0SetStudy::~study_rdt3_0SetStudy() {
  delete [] lpropValues;
  delete [] timeoutValues;
  delete [] upropValues;
  delete ThePVModel;
}


//******************************************************
// set values for lprop
//******************************************************
void study_rdt3_0SetStudy::SetValues_lprop() {
  lpropValues[0] = 0.5;
  lpropValues[1] = 0.4;
}


//******************************************************
// set values for timeout
//******************************************************
void study_rdt3_0SetStudy::SetValues_timeout() {
  timeoutValues[0] = 2.0;
  timeoutValues[1] = 2.0;
}


//******************************************************
// set values for uprop
//******************************************************
void study_rdt3_0SetStudy::SetValues_uprop() {
  upropValues[0] = 1.1;
  upropValues[1] = 1.2;
}



//******************************************************
//print values of gv (for debugging)
//******************************************************
void study_rdt3_0SetStudy::PrintGlobalValues(int expNum) {
  if (NumGVs == 0) {
    cout<<"There are no global variables."<<endl;
    return;
  }

  SetGVs(expNum);

  cout<<"The Global Variable values for experiment "<<
    GetExpName(expNum)<<" are:"<<endl;
  cout << "lprop\tdouble\t" << lprop << endl;
  cout << "timeout\tdouble\t" << timeout << endl;
  cout << "uprop\tdouble\t" << uprop << endl;
}


//******************************************************
//retrieve the value of a global variable
//******************************************************
void *study_rdt3_0SetStudy::GetGVValue(char *TheGVName) {
  if (strcmp("lprop", TheGVName) == 0)
    return &lprop;
  else if (strcmp("timeout", TheGVName) == 0)
    return &timeout;
  else if (strcmp("uprop", TheGVName) == 0)
    return &uprop;
  else 
    cerr<<"!! study_rdt3_0SetStudy::GetGVValue: Global Variable "<<TheGVName<<" does not exist."<<endl;
  return NULL;
}


//******************************************************
//override the value of a global variable
//******************************************************
void study_rdt3_0SetStudy::OverrideGVValue(char *TheGVName,void *TheGVValue) {
  if (strcmp("lprop", TheGVName) == 0)
    lprop = *(double *)TheGVValue;
  else if (strcmp("timeout", TheGVName) == 0)
    timeout = *(double *)TheGVValue;
  else if (strcmp("uprop", TheGVName) == 0)
    uprop = *(double *)TheGVValue;
  else 
    cerr<<"!! study_rdt3_0SetStudy::OverrideGVValue: Global Variable "<<TheGVName<<" does not exist."<<endl;
}


//******************************************************
//set the value of all global variables to the given exp
//******************************************************
void study_rdt3_0SetStudy::SetGVs(int expNum) {
  lprop = lpropValues[expNum];
  timeout = timeoutValues[expNum];
  uprop = upropValues[expNum];
}


//******************************************************
//static class method called by solvers to create study 
//(and thus create all of the model)
//******************************************************
BaseStudyClass* GlobalStudyPtr = NULL;
BaseStudyClass * GenerateStudy() {
  if (GlobalStudyPtr == NULL)
    GlobalStudyPtr = new study_rdt3_0SetStudy();
  return GlobalStudyPtr;
}

void DestructStudy() {
  delete GlobalStudyPtr;
  GlobalStudyPtr = NULL;
}
//******************************************************
//get and create the PVModel
//******************************************************
PVModel* study_rdt3_0SetStudy::GetPVModel() {
  // create the PV model
  ThePVModel=new reward_rdt3_0PVModel();
  return ThePVModel;
}


